<?php

class GitHub_Example {

	public function __construct() {
		if ( $_SERVER["REQUEST_METHOD"] == "GET" && isset($_GET['code'])){

			require_once('../../../wp-config.php');
			require_once(ABSPATH . 'wp-includes/plugin.php');
			require_once(ABSPATH . 'wp-includes/pluggable.php');
			require_once(ABSPATH . 'wp-includes/general-template.php');

			$redirect = get_template_directory_uri() . '/github.php';
			$api_key = get_option( 'GITHUB_API_KEY' );
			$api_secret = get_option( 'GITHUB_API_SECRET_KEY' );

			$args = array(
				'method' => 'POST',
				'httpversion' => '1.1',
				'blocking' => true,
				'headers' => array(
					'Accept' => 'application/json'
				),
				'body' => array( 
					'code' => $_GET['code'],
					'redirect_uri' => $redirect,
					'client_id' => $api_key,
					'client_secret' => $api_secret
				)
			);

			add_filter('https_ssl_verify', '__return_false');
			$response = wp_remote_post( 'https://github.com/login/oauth/access_token', $args );
			
			$keys = json_decode($response['body']);

			if($keys) {
				update_option( 'GITHUB_AUTHENTICATION_TOKEN', $keys->{'access_token'} );
			}
			wp_redirect( get_bloginfo( 'url' ) . '/wp-admin/options-general.php?page=github.php' ); 
			exit; 
		}

		add_shortcode('github', array($this, 'render_shortcode'));
		add_action('admin_menu', array($this, 'add_github_admin_menu'));
		add_action('wp_footer', array($this, 'enqueue_github_style'));
	}

	public function add_github_admin_menu() {  
		add_options_page('GitHub Options', 'GitHub', 'manage_options', 'github.php', array($this, 'add_github_admin_page'));
		add_action( 'admin_print_scripts', array($this, 'enqueue_github_style'));
	}  

	public function add_github_admin_page() {
		if ( !current_user_can( 'manage_options' ) )  {
			wp_die( __( 'You do not have sufficient permissions to access this page.' ) );
		}

		if ( $_SERVER["REQUEST_METHOD"] == "POST" ){
			update_option( 'GITHUB_API_KEY', $_POST['api_key'] );
			update_option( 'GITHUB_API_SECRET_KEY', $_POST['api_secret_key'] );
		}

		?>
		<div class="github-admin-options">
			<h1>GitHub Options</h1>
			<form name="options" method="POST" action="<?php echo $_SERVER['REQUEST_URI']; ?>">
				<label for="api_key">API Key<span class="required">(*)</span>: </label>
				<input type="text" name="api_key" value="<?php echo get_option( 'GITHUB_API_KEY', '' ); ?>" size="70">
				<br />
				<label for="api_secret_key">Consumer Secret<span class="required">(*)</span>: </label>
				<input type="text" name="api_secret_key" value="<?php echo get_option( 'GITHUB_API_SECRET_KEY', '' ); ?>" size="70">
				<br />
				<label for="bearer_token">Authentication Token: </label>
				<input type="text" disabled value="<?php echo get_option( 'GITHUB_AUTHENTICATION_TOKEN', '' ); ?>" size="70">
				<br />
				<input class="button-primary" type="submit" name="save" />

				<?php

				$state = base64_encode(time());
				$redirect = get_template_directory_uri() . '/github.php';
				$api_key = get_option( 'GITHUB_API_KEY' );
				$api_secret = get_option( 'GITHUB_API_SECRET_KEY' );
				$token = get_option( 'GITHUB_AUTHENTICATION_TOKEN' );

				if($api_key && $api_secret && !$token) {
					$api_url = "https://github.com/login/oauth/authorize?client_id=$api_key&scope=&state=$state&redirect_uri=$redirect";
					?>
					<a class="button-primary" type="button" href="<?php echo $api_url; ?>">Authenticate</a>
					<?php
				}

				?>
				<br/>
				<small>You can sign up for a API key <a href="https://developer.github.com/" target="_blank">here</a></small>
			</form>
			<br />
			<?php echo do_shortcode('[github]'); ?>
		</div>
		<?php
	}
	

	public function render_shortcode($atts) {
		$return = '';

		extract( shortcode_atts( array(
			'id' => 'github-1',
		), $atts ) );

		$token = get_option( 'GITHUB_AUTHENTICATION_TOKEN' );

		if($token) {
			add_filter('https_ssl_verify', '__return_false');
			
			$api_url = "https://api.github.com/user?access_token=$token";
			$response = wp_remote_get( $api_url );
			$json = json_decode( $response['body'] );
		
			$return .= '<div class="github">';
			$return .= '<h2><a href="' . $json->{'html_url'} . '" target="_blank">' . $json->{'name'} . '</a></h2>';
			$return .= '<div class="bio">';
			$return .= '<span>' . $json->{'bio'} . '</span>';
			$return .= '</div>';
			$return .= '<div class="counts">';
			$return .= '<div>' . $json->{'public_repos'} . '<br/><span>repos</span></div>';	
			$return .= '<div>' . $json->{'followers'} . '<br/><span>followers</span></div>';
			$return .= '<div>' . $json->{'following'} . '<br/><span>following</span></div>';
			$return .= '</div>';
			
			$api_url = "https://api.github.com/user/repos?access_token=$token";
			$response = wp_remote_get( $api_url );
			$json = json_decode( $response['body'] );

			$return .= '<ol class="repos">';
			foreach($json as $i => $repos) {
				$return .= '<li>';
				$return .= '<h3><a href="' . $repos->{'html_url'} . '" target="_blank" title="' . $repos->{'description'} . '">' . str_replace('_', ' ', $repos->{'name'} ) . '</a></h3>';
				$return .= '<div>';
				$return .= '<div>';
				$return .= $repos->{'forks_count'};
				$return .= '<br/><span>forks</span>';
				$return .= '</div>';
				$return .= '<div>';
				$return .= $repos->{'watchers_count'};
				$return .= '<br/><span>watchers</span>';
				$return .= '</div>';
				$return .= '</div>';
				$return .= '</li>';
			}
			$return .= '</ol>';
			$return .= '</div>';
			
			return $return;
		}
	}

	public function enqueue_github_style() {
		$token = get_option( 'GITHUB_AUTHENTICATION_TOKEN' );

		if($token) {
			 wp_register_style( 'github-style', get_template_directory_uri() . '/github.css');

			 wp_enqueue_style( 'github-style' );
		}
	}
}

$github = new GitHub_Example();

?>